@@ -15,9 +15,11 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 # MA 02110-1301 USA.
 
+from __future__ import print_function
+
 import glob
+import io
 import os
-import StringIO
 import unittest
 
 from virtconv import VirtConverter
@@ -26,15 +28,15 @@ from tests import utils
 
 base_dir = os.getcwd() + "/tests/virtconv-files/"
 out_dir = base_dir + "libvirt_output"
-conn = utils.open_kvm()
 
 
 class TestVirtConv(unittest.TestCase):
     def _convert_helper(self, infile, outfile, in_type, disk_format):
-        outbuf = StringIO.StringIO()
+        outbuf = io.BytesIO()
         def print_cb(msg):
-            print >> outbuf, msg
+            print(msg, file=outbuf)
 
+        conn = utils.open_kvm()
         converter = VirtConverter(conn, infile, print_cb=print_cb)
 
         if converter.parser.name != in_type:
@@ -55,7 +57,7 @@ class TestVirtConv(unittest.TestCase):
             self.skipTest("Not comparing XML because vmport isn't supported")
 
         utils.diff_compare(out_expect, outfile)
-        utils.test_create(converter.conn, out_xml)
+        utils.test_create(conn, out_xml)
 
     def _compare_single_file(self, in_path, in_type, disk_format=None):
         cwd = os.getcwd()
