@@ -25,6 +25,8 @@ from gi.repository import Gtk
 from gi.repository import Gdk
 from gi.repository import GdkPixbuf
 
+import libvirt
+
 from virtinst import util
 
 from . import vmmenu
@@ -32,8 +34,6 @@ from . import uiutil
 from .baseclass import vmmGObjectUI
 from .graphwidgets import CellRendererSparkline
 
-import libvirt
-
 # Number of data points for performance graphs
 GRAPH_LEN = 40
 
@@ -66,6 +66,10 @@ def _style_get_prop(widget, propname):
     return value.get_int()
 
 
+def _cmp(a, b):
+    return ((a > b) - (a < b))
+
+
 def _get_inspection_icon_pixbuf(vm, w, h):
     # libguestfs gives us the PNG data as a string.
     png_data = vm.inspection.icon
@@ -78,7 +82,7 @@ def _get_inspection_icon_pixbuf(vm, w, h):
         pb.write(png_data)
         pb.close()
         return pb.get_pixbuf()
-    except:
+    except Exception:
         logging.exception("Error loading inspection icon data")
         vm.inspection.icon = None
         return None
@@ -127,15 +131,15 @@ class vmmManager(vmmGObjectUI):
 
         self.builder.connect_signals({
             "on_menu_view_guest_cpu_usage_activate":
-                    self.toggle_stats_visible_guest_cpu,
+            self.toggle_stats_visible_guest_cpu,
             "on_menu_view_host_cpu_usage_activate":
-                    self.toggle_stats_visible_host_cpu,
+            self.toggle_stats_visible_host_cpu,
             "on_menu_view_memory_usage_activate":
-                    self.toggle_stats_visible_memory_usage,
-            "on_menu_view_disk_io_activate" :
-                    self.toggle_stats_visible_disk,
+            self.toggle_stats_visible_memory_usage,
+            "on_menu_view_disk_io_activate":
+            self.toggle_stats_visible_disk,
             "on_menu_view_network_traffic_activate":
-                    self.toggle_stats_visible_network,
+            self.toggle_stats_visible_network,
 
             "on_vm_manager_delete_event": self.close,
             "on_vmm_manager_configure_event": self.window_resized,
@@ -311,6 +315,7 @@ class vmmManager(vmmGObjectUI):
                 item.set_image(icon)
             if cb:
                 item.connect("activate", cb)
+            item.get_accessible().set_name("conn-%s" % idx)
             self.connmenu.add(item)
             self.connmenu_items[idx] = item
 
@@ -458,10 +463,10 @@ class vmmManager(vmmGObjectUI):
     # Action listeners #
     ####################
 
-    def window_resized(self, ignore, event):
+    def window_resized(self, ignore, ignore2):
         if not self.is_visible():
             return
-        self._window_size = (event.width, event.height)
+        self._window_size = self.topwin.get_size()
 
     def exit_app(self, src_ignore=None, src2_ignore=None):
         self.emit("action-exit-app")
@@ -579,17 +584,26 @@ class vmmManager(vmmGObjectUI):
 
     def vm_added(self, conn, connkey):
         vm = conn.get_vm(connkey)
-        if not vm or self.vm_row_key(vm) in self.rows:
+        if not vm:
             return
 
+        row_key = self.vm_row_key(vm)
+        if row_key in self.rows:
+            return
+
+        row = self._build_row(None, vm)
+        parent = self.rows[conn.get_uri()].iter
+        model = self.widget("vm-list").get_model()
+        _iter = model.append(parent, row)
+        path = model.get_path(_iter)
+        self.rows[row_key] = model[path]
+
         vm.connect("state-changed", self.vm_changed)
         vm.connect("resources-sampled", self.vm_row_updated)
         vm.connect("inspection-changed", self.vm_inspection_changed)
 
-        vmlist = self.widget("vm-list")
-        model = vmlist.get_model()
-
-        self._append_vm(model, vm, conn)
+        # Expand a connection when adding a vm to it
+        self.widget("vm-list").expand_row(model.get_path(parent), False)
 
     def vm_removed(self, conn, connkey):
         vmlist = self.widget("vm-list")
@@ -667,29 +681,6 @@ class vmmManager(vmmGObjectUI):
 
         return row
 
-    def _append_vm(self, model, vm, conn):
-        row_key = self.vm_row_key(vm)
-        if row_key in self.rows:
-            return
-
-        row = self._build_row(None, vm)
-        parent = self.rows[conn.get_uri()].iter
-
-        _iter = model.append(parent, row)
-        path = model.get_path(_iter)
-        self.rows[row_key] = model[path]
-
-        # Expand a connection when adding a vm to it
-        self.widget("vm-list").expand_row(model.get_path(parent), False)
-
-    def _append_conn(self, model, conn):
-        row = self._build_row(conn, None)
-
-        _iter = model.append(None, row)
-        path = model.get_path(_iter)
-        self.rows[conn.get_uri()] = model[path]
-        return _iter
-
     def add_conn(self, engine_ignore, conn):
         # Called from engine.py signal conn-added
 
@@ -699,14 +690,17 @@ class vmmManager(vmmGObjectUI):
         if conn.get_uri() in self.rows:
             return
 
+        model = self.widget("vm-list").get_model()
+        row = self._build_row(conn, None)
+        _iter = model.append(None, row)
+        path = model.get_path(_iter)
+        self.rows[conn.get_uri()] = model[path]
+
         conn.connect("vm-added", self.vm_added)
         conn.connect("vm-removed", self.vm_removed)
         conn.connect("resources-sampled", self.conn_row_updated)
         conn.connect("state-changed", self.conn_state_changed)
 
-        vmlist = self.widget("vm-list")
-        self._append_conn(vmlist.get_model(), conn)
-
     def remove_conn(self, engine_ignore, uri):
         # Called from engine.py signal conn-removed
 
@@ -754,10 +748,8 @@ class vmmManager(vmmGObjectUI):
             row[ROW_MARKUP] = self._build_vm_markup(name, status)
 
             desc = vm.get_description()
-            if not uiutil.can_set_row_none:
-                desc = desc or ""
             row[ROW_HINT] = util.xml_escape(desc)
-        except libvirt.libvirtError, e:
+        except libvirt.libvirtError as e:
             if util.exception_is_libvirt_error(e, "VIR_ERR_NO_DOMAIN"):
                 return
             raise
@@ -770,8 +762,6 @@ class vmmManager(vmmGObjectUI):
             return
 
         new_icon = _get_inspection_icon_pixbuf(vm, 16, 16)
-        if not uiutil.can_set_row_none:
-            new_icon = new_icon or ""
         row[ROW_INSPECTION_OS_ICON] = new_icon
 
         self.vm_row_updated(vm)
@@ -926,42 +916,43 @@ class vmmManager(vmmGObjectUI):
     def vmlist_name_sorter(self, model, iter1, iter2, ignore):
         key1 = str(model[iter1][ROW_SORT_KEY]).lower()
         key2 = str(model[iter2][ROW_SORT_KEY]).lower()
-        return cmp(key1, key2)
+        return _cmp(key1, key2)
 
     def vmlist_guest_cpu_usage_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
         obj2 = model[iter2][ROW_HANDLE]
 
-        return cmp(obj1.guest_cpu_time_percentage(),
+        return _cmp(obj1.guest_cpu_time_percentage(),
                    obj2.guest_cpu_time_percentage())
 
     def vmlist_host_cpu_usage_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
         obj2 = model[iter2][ROW_HANDLE]
 
-        return cmp(obj1.host_cpu_time_percentage(),
+        return _cmp(obj1.host_cpu_time_percentage(),
                    obj2.host_cpu_time_percentage())
 
     def vmlist_memory_usage_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
         obj2 = model[iter2][ROW_HANDLE]
 
-        return cmp(obj1.stats_memory(),
+        return _cmp(obj1.stats_memory(),
                    obj2.stats_memory())
 
     def vmlist_disk_io_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
         obj2 = model[iter2][ROW_HANDLE]
 
-        return cmp(obj1.disk_io_rate(), obj2.disk_io_rate())
+        return _cmp(obj1.disk_io_rate(), obj2.disk_io_rate())
 
     def vmlist_network_usage_sorter(self, model, iter1, iter2, ignore):
         obj1 = model[iter1][ROW_HANDLE]
         obj2 = model[iter2][ROW_HANDLE]
 
-        return cmp(obj1.network_traffic_rate(), obj2.network_traffic_rate())
+        return _cmp(obj1.network_traffic_rate(), obj2.network_traffic_rate())
 
     def enable_polling(self, column):
+        # pylint: disable=redefined-variable-type
         if column == COL_GUEST_CPU:
             widgn = ["menu_view_stats_guest_cpu", "menu_view_stats_host_cpu"]
             do_enable = self.config.get_stats_enable_cpu_poll()
