@@ -62,7 +62,7 @@ class vmmNetworkList(vmmGObjectUI):
             self.conn.disconnect_by_func(self._repopulate_network_list)
             self.conn.disconnect_by_func(self._repopulate_network_list)
             self.conn.disconnect_by_func(self._repopulate_network_list)
-        except:
+        except Exception:
             pass
 
         self.conn = None
@@ -90,10 +90,10 @@ class vmmNetworkList(vmmGObjectUI):
         combo.set_model(model)
         uiutil.init_combo_text_column(combo, 1)
 
-        model.append(["bridge", "Bridge"])
+        model.append(["bridge", _("Bridge")])
         model.append(["vepa", "VEPA"])
-        model.append(["private", "Private"])
-        model.append(["passthrough", "Passthrough"])
+        model.append(["private", _("Private")])
+        model.append(["passthrough", _("Passthrough")])
         combo.set_active(0)
 
         combo = self.widget("net-portgroup")
@@ -129,7 +129,7 @@ class vmmNetworkList(vmmGObjectUI):
         return ret
 
     def _build_source_row(self, nettype, source_name,
-        label, is_sensitive, is_running, manual_bridge=False, key=None):
+            label, is_sensitive, is_running, manual_bridge=False, key=None):
         return [nettype, source_name, label,
                 is_sensitive, is_running, manual_bridge,
                 key]
@@ -146,6 +146,9 @@ class vmmNetworkList(vmmGObjectUI):
             if not net.is_active():
                 label += " (%s)" % _("Inactive")
 
+            if net.get_xmlobj().virtualport_type == "openvswitch":
+                label += " (OpenVSwitch)"
+
             if net.get_name() == "default":
                 default_label = label
 
@@ -235,18 +238,22 @@ class vmmNetworkList(vmmGObjectUI):
 
         return rows, default_label
 
-    def _populate_network_list(self):
-        net_list = self.widget("net-source")
-        model = net_list.get_model()
+    def _populate_network_model(self, model):
         model.clear()
 
-        # For qemu:///session
+        def _add_manual_bridge_row():
+            manual_row = self._build_source_row(
+                None, None, _("Specify shared device name"),
+                True, False, manual_bridge=True)
+            model.append(manual_row)
+
         if self.conn.is_qemu_session():
             nettype = virtinst.VirtualNetworkInterface.TYPE_USER
             r = self._build_source_row(
                 nettype, None, self._pretty_network_desc(nettype), True, True)
             model.append(r)
-            net_list.set_active(0)
+
+            _add_manual_bridge_row()
             return
 
         (vnets, vnet_bridges, default_net) = self._find_virtual_networks()
@@ -281,16 +288,11 @@ class vmmNetworkList(vmmGObjectUI):
             model.insert(0, row)
             default = 0
         elif label:
-            default = [idx for idx in range(len(model)) if
-                       model[idx][2] == label][0]
+            default = [idx for idx, model_label in enumerate(model) if
+                       model_label[2] == label][0]
 
-        # After all is said and done, add a manual bridge option
-        manual_row = self._build_source_row(
-            None, None, _("Specify shared device name"),
-            True, False, manual_bridge=True)
-        model.append(manual_row)
-
-        net_list.set_active(default)
+        _add_manual_bridge_row()
+        return default
 
 
     ###############
@@ -298,7 +300,7 @@ class vmmNetworkList(vmmGObjectUI):
     ###############
 
     def get_network_row(self):
-        return uiutil.get_list_selection(self.widget("net-source"), None)
+        return uiutil.get_list_selected_row(self.widget("net-source"))
 
     def get_network_selection(self):
         bridge_entry = self.widget("net-bridge-name")
@@ -312,15 +314,15 @@ class vmmNetworkList(vmmGObjectUI):
 
         if net_check_bridge and bridge_entry:
             net_type = virtinst.VirtualNetworkInterface.TYPE_BRIDGE
-            net_src = bridge_entry.get_text()
+            net_src = bridge_entry.get_text() or None
 
         mode = None
         if self.widget("net-source-mode").is_visible():
-            mode = uiutil.get_combo_entry(self.widget("net-source-mode"), 0)
+            mode = uiutil.get_list_selection(self.widget("net-source-mode"))
 
         portgroup = None
         if self.widget("net-portgroup").is_visible():
-            portgroup = uiutil.get_combo_entry(self.widget("net-portgroup"), 0)
+            portgroup = uiutil.get_list_selection(self.widget("net-portgroup"))
 
         return net_type, net_src, mode, portgroup or None
 
@@ -361,7 +363,7 @@ class vmmNetworkList(vmmGObjectUI):
             try:
                 netobj.start()
                 logging.info("Started network '%s'", devname)
-            except Exception, e:
+            except Exception as e:
                 return self.err.show_err(_("Could not start virtual network "
                                       "'%s': %s") % (devname, str(e)))
 
@@ -386,7 +388,7 @@ class vmmNetworkList(vmmGObjectUI):
                 net.virtualport.typeid = vport_typeid or None
                 net.virtualport.typeidversion = vport_idver or None
                 net.virtualport.instanceid = vport_instid or None
-        except Exception, e:
+        except Exception as e:
             return self.err.val_err(_("Error with network parameters."), e)
 
         # Make sure there is no mac address collision
@@ -402,7 +404,7 @@ class vmmNetworkList(vmmGObjectUI):
         return net
 
     def reset_state(self):
-        self._populate_network_list()
+        self._repopulate_network_list()
 
         net_err = None
         if (not self.conn.is_nodedev_capable() or
@@ -432,7 +434,7 @@ class vmmNetworkList(vmmGObjectUI):
         source_mode = net.source_mode
         is_direct = (net.type == "direct")
 
-        uiutil.set_combo_entry(self.widget("net-source-mode"), source_mode)
+        uiutil.set_list_selection(self.widget("net-source-mode"), source_mode)
 
         # Virtualport config
         self.widget("vport-expander").set_visible(is_direct)
@@ -466,7 +468,7 @@ class vmmNetworkList(vmmGObjectUI):
         combo.emit("changed")
 
         if net.portgroup:
-            uiutil.set_combo_entry(self.widget("net-portgroup"), net.portgroup)
+            uiutil.set_list_selection(self.widget("net-portgroup"), net.portgroup)
 
 
     #############
@@ -474,28 +476,39 @@ class vmmNetworkList(vmmGObjectUI):
     #############
 
     def _emit_changed(self, *args, **kwargs):
-        ignore = args
-        ignore = kwargs
+        ignore1 = args
+        ignore2 = kwargs
         self.emit("changed")
 
     def _emit_vport_changed(self, *args, **kwargs):
-        ignore = args
-        ignore = kwargs
+        ignore1 = args
+        ignore2 = kwargs
         self.emit("changed-vport")
 
     def _repopulate_network_list(self, *args, **kwargs):
-        ignore = args
-        ignore = kwargs
+        ignore1 = args
+        ignore2 = kwargs
 
         netlist = self.widget("net-source")
-        label = uiutil.get_list_selection(netlist, 2)
-        self._populate_network_list()
+        current_label = uiutil.get_list_selection(netlist, column=2)
+
+        model = netlist.get_model()
+        try:
+            netlist.set_model(None)
+            default_idx = self._populate_network_model(model)
+        finally:
+            netlist.set_model(model)
 
         for row in netlist.get_model():
-            if label and row[2] == label:
+            if current_label and row[2] == current_label:
                 netlist.set_active_iter(row.iter)
                 return
 
+        if default_idx is None:
+            default_idx = 0
+        netlist.set_active(default_idx)
+
+
     def _populate_portgroups(self, portgroups):
         combo = self.widget("net-portgroup")
         model = combo.get_model()
@@ -507,7 +520,7 @@ class vmmNetworkList(vmmGObjectUI):
             if p.default:
                 default = p.name
 
-        uiutil.set_combo_entry(combo, default)
+        uiutil.set_list_selection(combo, default)
 
     def _on_net_source_changed(self, src):
         ignore = src
@@ -516,8 +529,9 @@ class vmmNetworkList(vmmGObjectUI):
         if not row:
             return
 
+        is_openvswitch = row[2].endswith("(OpenVSwitch)")
         is_direct = (row[0] == virtinst.VirtualNetworkInterface.TYPE_DIRECT)
-        self.widget("vport-expander").set_visible(is_direct)
+        self.widget("vport-expander").set_visible(is_direct or is_openvswitch)
         uiutil.set_grid_row_visible(self.widget("net-source-mode"), is_direct)
         uiutil.set_grid_row_visible(
             self.widget("net-macvtap-warn-box"), is_direct)
