@@ -19,18 +19,22 @@
 # MA 02110-1301 USA.
 #
 
+import logging
+import socket
+
 from gi.repository import GObject
 from gi.repository import Gdk
 
 import gi
 gi.require_version('GtkVnc', '2.0')
 from gi.repository import GtkVnc
-gi.require_version('SpiceClientGtk', '3.0')
-from gi.repository import SpiceClientGtk
-from gi.repository import SpiceClientGLib
-
-import logging
-import socket
+try:
+    gi.require_version('SpiceClientGtk', '3.0')
+    from gi.repository import SpiceClientGtk
+    from gi.repository import SpiceClientGLib
+    have_spice_gtk = True
+except (ValueError, ImportError):
+    have_spice_gtk = False
 
 from .baseclass import vmmGObject
 from .sshtunnels import SSHTunnels
@@ -52,7 +56,7 @@ class Viewer(vmmGObject):
         "pointer-grab": (GObject.SignalFlags.RUN_FIRST, None, []),
         "pointer-ungrab": (GObject.SignalFlags.RUN_FIRST, None, []),
         "connected": (GObject.SignalFlags.RUN_FIRST, None, []),
-        "disconnected": (GObject.SignalFlags.RUN_FIRST, None, []),
+        "disconnected": (GObject.SignalFlags.RUN_FIRST, None, [str, str]),
         "auth-error": (GObject.SignalFlags.RUN_FIRST, None, [str, bool]),
         "auth-rejected": (GObject.SignalFlags.RUN_FIRST, None, [str]),
         "need-auth": (GObject.SignalFlags.RUN_FIRST, None, [bool, bool]),
@@ -60,9 +64,10 @@ class Viewer(vmmGObject):
         "usb-redirect-error": (GObject.SignalFlags.RUN_FIRST, None, [str]),
     }
 
-    def __init__(self, ginfo):
+    def __init__(self, vm, ginfo):
         vmmGObject.__init__(self)
         self._display = None
+        self._vm = vm
         self._ginfo = ginfo
         self._tunnels = SSHTunnels(self._ginfo)
 
@@ -80,6 +85,7 @@ class Viewer(vmmGObject):
         if self._display:
             self._display.destroy()
         self._display = None
+        self._vm = None
 
         self._tunnels.close_all()
 
@@ -130,15 +136,41 @@ class Viewer(vmmGObject):
     def _get_pixbuf(self):
         return self._display.get_pixbuf()
 
-    def _open(self):
+    def _get_fd_for_open(self):
         if self._ginfo.need_tunnel():
-            self._open_fd(self._tunnels.open_new())
+            return self._tunnels.open_new()
+
+        if self._vm.conn.is_remote():
+            # OpenGraphics only works for local libvirtd connections
+            return None
+
+        if self._ginfo.gtlsport and not self._ginfo.gport:
+            # This makes spice loop requesting an fd. Disable until spice is
+            # fixed: https://bugzilla.redhat.com/show_bug.cgi?id=1334071
+            return None
+
+        if not self._vm.conn.check_support(
+                self._vm.conn.SUPPORT_DOMAIN_OPEN_GRAPHICS):
+            return None
+
+        return self._vm.open_graphics_fd()
+
+    def _open(self):
+        if self._ginfo.bad_config():
+            raise RuntimeError(self._ginfo.bad_config())
+
+        fd = self._get_fd_for_open()
+        if fd is not None:
+            self._open_fd(fd)
         else:
             self._open_host()
 
     def _get_grab_keys(self):
         return self._display.get_grab_keys().as_string()
 
+    def _emit_disconnected(self, errdetails=None):
+        ssherr = self._tunnels.get_err_output()
+        self.emit("disconnected", errdetails, ssherr)
 
 
     #######################################################
@@ -174,7 +206,7 @@ class Viewer(vmmGObject):
 
     def _get_scaling(self):
         raise NotImplementedError()
-    def _set_scaling(self, val):
+    def _set_scaling(self, scaling):
         raise NotImplementedError()
 
     def _set_resizeguest(self, val):
@@ -222,14 +254,18 @@ class Viewer(vmmGObject):
     def console_send_keys(self, keys):
         return self._send_keys(keys)
 
-    def console_get_err_output(self):
-        return self._tunnels.get_err_output()
-
     def console_get_grab_keys(self):
         return self._get_grab_keys()
 
     def console_get_desktop_resolution(self):
-        return self._get_desktop_resolution()
+        ret = self._get_desktop_resolution()
+        if not ret:
+            return ret
+
+        # Don't pass on bogus resolutions
+        if (ret[0] == 0) or (ret[1] == 0):
+            return None
+        return ret
 
     def console_get_scaling(self):
         return self._get_scaling()
@@ -300,7 +336,7 @@ class VNCViewer(Viewer):
 
     def _disconnected_cb(self, ignore):
         self._tunnels.unlock()
-        self.emit("disconnected")
+        self._emit_disconnected()
 
     def _desktop_resize(self, src_ignore, w, h):
         self._desktop_resolution = (w, h)
@@ -324,9 +360,8 @@ class VNCViewer(Viewer):
 
             errmsg = (_("Unable to provide requested credentials to the VNC "
                 "server.\n The credential type %s is not supported") %
-                str(cred))
+                str(cred.value_name))
 
-            # XXX test this
             self.emit("auth-rejected", errmsg)
             return
 
@@ -377,14 +412,14 @@ class VNCViewer(Viewer):
 
             try:
                 keys = [int(k) for k in keys.split(',')]
-            except:
+            except Exception:
                 logging.debug("Error in grab_keys configuration in Gsettings",
                               exc_info=True)
                 return
 
             seq = GtkVnc.GrabSequence.new(keys)
             self._display.set_grab_keys(seq)
-        except Exception, e:
+        except Exception as e:
             logging.debug("Error when getting the grab keys combination: %s",
                           str(e))
 
@@ -421,15 +456,15 @@ class VNCViewer(Viewer):
     # Connection routines #
     #######################
 
-    def _open(self, *args, **kwargs):
+    def _open(self):
         self._init_widget()
-        return Viewer._open(self, *args, **kwargs)
+        return Viewer._open(self)
 
     def _open_host(self):
         host, port, ignore = self._ginfo.get_conn_host()
 
         if not self._ginfo.gsocket:
-            logging.debug("VNC connection to %s:%s", host, port)
+            logging.debug("VNC connecting to host=%s port=%s", host, port)
             self._display.open_host(host, port)
             return
 
@@ -438,7 +473,7 @@ class VNCViewer(Viewer):
             sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
             sock.connect(self._ginfo.gsocket)
             self._sockfd = sock
-        except Exception, e:
+        except Exception as e:
             raise RuntimeError(_("Error opening socket path '%s': %s") %
                                (self._ginfo.gsocket, e))
 
@@ -503,16 +538,23 @@ class SpiceViewer(Viewer):
         GObject.GObject.connect(self._spice_session, "channel-new",
                                 self._channel_new_cb)
 
-        self._usbdev_manager = SpiceClientGLib.UsbDeviceManager.get(
-                                    self._spice_session)
-        self._usbdev_manager.connect("auto-connect-failed",
-                                    self._usbdev_redirect_error)
-        self._usbdev_manager.connect("device-error",
-                                    self._usbdev_redirect_error)
+        # Distros might have usb redirection compiled out, like OpenBSD
+        # https://bugzilla.redhat.com/show_bug.cgi?id=1348479
+        try:
+            self._usbdev_manager = SpiceClientGLib.UsbDeviceManager.get(
+                                        self._spice_session)
+            self._usbdev_manager.connect("auto-connect-failed",
+                                        self._usbdev_redirect_error)
+            self._usbdev_manager.connect("device-error",
+                                        self._usbdev_redirect_error)
 
-        autoredir = self.config.get_auto_redirection()
-        if autoredir:
-            gtk_session.set_property("auto-usbredir", True)
+            autoredir = self.config.get_auto_redirection()
+            if autoredir:
+                gtk_session.set_property("auto-usbredir", True)
+        except Exception:
+            self._usbdev_manager = None
+            logging.debug("Error initializing spice usb device manager",
+                exc_info=True)
 
 
     #####################
@@ -520,8 +562,10 @@ class SpiceViewer(Viewer):
     #####################
 
     def _main_channel_event_cb(self, channel, event):
+        self._tunnels.unlock()
+
         if event == SpiceClientGLib.ChannelEvent.CLOSED:
-            self.emit("disconnected")
+            self._emit_disconnected()
         elif event == SpiceClientGLib.ChannelEvent.ERROR_AUTH:
             if not self._spice_session.get_property("password"):
                 logging.debug("Spice channel received ERROR_AUTH, but no "
@@ -531,12 +575,21 @@ class SpiceViewer(Viewer):
                 logging.debug("Spice channel received ERROR_AUTH, but a "
                     "password is already set. Assuming authentication failed.")
                 self.emit("auth-error", channel.get_error().message, False)
-        elif event in [SpiceClientGLib.ChannelEvent.ERROR_CONNECT,
-                       SpiceClientGLib.ChannelEvent.ERROR_IO,
-                       SpiceClientGLib.ChannelEvent.ERROR_LINK,
-                       SpiceClientGLib.ChannelEvent.ERROR_TLS]:
-            logging.debug("Spice channel event error: %s", event)
-            self.emit("disconnected")
+        elif "ERROR" in str(event):
+            # SpiceClientGLib.ChannelEvent.ERROR_CONNECT
+            # SpiceClientGLib.ChannelEvent.ERROR_IO
+            # SpiceClientGLib.ChannelEvent.ERROR_LINK
+            # SpiceClientGLib.ChannelEvent.ERROR_TLS
+            error = None
+            if channel.get_error():
+                error = channel.get_error().message
+            logging.debug("Spice channel event=%s message=%s", event, error)
+
+            msg = _("Encountered SPICE %(error-name)s") % {
+                "error-name": event.value_nick}
+            if error:
+                msg += ": %s" % error
+            self._emit_disconnected(msg)
 
     def _fd_channel_event_cb(self, channel, event):
         # When we see any event from the channel, release the
@@ -551,19 +604,18 @@ class SpiceViewer(Viewer):
             # are still rolling in
             return
 
-        logging.debug("Requesting tunnel for channel: %s", channel)
+        logging.debug("Requesting fd for channel: %s", channel)
         channel.connect_after("channel-event", self._fd_channel_event_cb)
 
-        fd = self._tunnels.open_new()
+        fd = self._get_fd_for_open()
         channel.open_fd(fd)
 
     def _channel_new_cb(self, session, channel):
         GObject.GObject.connect(channel, "open-fd",
                                 self._channel_open_fd_request)
 
-        if (type(channel) == SpiceClientGLib.MainChannel and
+        if (isinstance(channel, SpiceClientGLib.MainChannel) and
             not self._main_channel):
-            self._tunnels.unlock()
             self._main_channel = channel
             hid = self._main_channel.connect_after("channel-event",
                 self._main_channel_event_cb)
@@ -573,7 +625,7 @@ class SpiceViewer(Viewer):
             self._main_channel_hids.append(hid)
 
         elif (type(channel) == SpiceClientGLib.DisplayChannel and
-            not self._display):
+                not self._display):
             channel_id = channel.get_property("channel-id")
 
             if channel_id != 0:
@@ -588,7 +640,7 @@ class SpiceViewer(Viewer):
 
         elif (type(channel) in [SpiceClientGLib.PlaybackChannel,
                                 SpiceClientGLib.RecordChannel] and
-            not self._audio):
+                                not self._audio):
             self._audio = SpiceClientGLib.Audio.get(self._spice_session, None)
 
     def _agent_connected_cb(self, src, val):
@@ -626,14 +678,14 @@ class SpiceViewer(Viewer):
 
             try:
                 keys = [int(k) for k in keys.split(',')]
-            except:
+            except Exception:
                 logging.debug("Error in grab_keys configuration in Gsettings",
                               exc_info=True)
                 return
 
             seq = SpiceClientGtk.GrabSequence.new(keys)
             self._display.set_grab_keys(seq)
-        except Exception, e:
+        except Exception as e:
             logging.debug("Error when getting the grab keys combination: %s",
                           str(e))
 
@@ -661,6 +713,8 @@ class SpiceViewer(Viewer):
         host, port, tlsport = self._ginfo.get_conn_host()
         self._create_spice_session()
 
+        logging.debug("Spice connecting to host=%s port=%s tlsport=%s",
+            host, port, tlsport)
         self._spice_session.set_property("host", str(host))
         if port:
             self._spice_session.set_property("port", str(port))
@@ -673,12 +727,13 @@ class SpiceViewer(Viewer):
         self._create_spice_session()
         self._spice_session.open_fd(fd)
 
-    def _set_username(self, val):
-        ignore = val
+    def _set_username(self, cred):
+        ignore = cred
     def _set_password(self, cred):
         self._spice_session.set_property("password", cred)
-        if self._ginfo.need_tunnel():
-            self._spice_session.open_fd(self._tunnels.open_new())
+        fd = self._get_fd_for_open()
+        if fd is not None:
+            self._spice_session.open_fd(fd)
         else:
             self._spice_session.connect()
 
