@@ -24,8 +24,8 @@ import logging
 
 import libvirt
 
-from virtinst import util
-from virtinst.xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
+from . import util
+from .xmlbuilder import XMLBuilder, XMLChildProperty, XMLProperty
 
 
 class _NetworkDHCPRange(XMLBuilder):
@@ -58,7 +58,7 @@ class _NetworkIP(XMLBuilder):
 
     def add_range(self):
         r = _NetworkDHCPRange(self.conn)
-        self._add_child(r)
+        self.add_child(r)
         return r
 
 
@@ -72,11 +72,33 @@ class _NetworkRoute(XMLBuilder):
     netmask = XMLProperty("./@netmask")
 
 
+class _NetworkForwardPf(XMLBuilder):
+    _XML_ROOT_NAME = "pf"
+    dev = XMLProperty("./@dev")
+
+
+class _NetworkForwardAddress(XMLBuilder):
+    _XML_ROOT_NAME = "address"
+    type = XMLProperty("./@type")
+    domain = XMLProperty("./@domain", is_int=True)
+    bus = XMLProperty("./@bus", is_int=True)
+    slot = XMLProperty("./@slot", is_int=True)
+    function = XMLProperty("./@function", is_int=True)
+
+
 class _NetworkForward(XMLBuilder):
     _XML_ROOT_NAME = "forward"
 
     mode = XMLProperty("./@mode")
     dev = XMLProperty("./@dev")
+    managed = XMLProperty("./@managed")
+    pf = XMLChildProperty(_NetworkForwardPf)
+    vfs = XMLChildProperty(_NetworkForwardAddress)
+
+    def add_pf(self):
+        r = _NetworkForwardPf(self.conn)
+        self.add_child(r)
+        return r
 
     def pretty_desc(self):
         return Network.pretty_forward_desc(self.mode, self.dev)
@@ -95,12 +117,12 @@ class _NetworkBandwidth(XMLBuilder):
     outbound_burst = XMLProperty("./outbound/@burst")
 
     def is_inbound(self):
-        return bool(self.inbound_average or self.inbound_peak
-                    or self.inbound_burst or self.inbound_floor)
+        return bool(self.inbound_average or self.inbound_peak or
+                    self.inbound_burst or self.inbound_floor)
 
     def is_outbound(self):
-        return bool(self.outbound_average or self.outbound_peak
-                    or self.outbound_burst)
+        return bool(self.outbound_average or self.outbound_peak or
+                    self.outbound_burst)
 
     def pretty_desc(self, inbound=True, outbound=True):
         items_in = [(self.inbound_average, _("Average"), "KiB/s"),
@@ -160,18 +182,15 @@ class Network(XMLBuilder):
                     desc = _("Routed network")
             else:
                 if dev:
-                    desc = "%s to %s" % (mode, dev)
+                    desc = (_("%(mode)s to %(device)s") %
+                            {"mode": mode, "device": dev})
                 else:
-                    desc = "%s network" % mode.capitalize()
+                    desc = _("%s network") % mode.capitalize()
         else:
             desc = _("Isolated network, internal and host routing only")
 
         return desc
 
-    def __init__(self, *args, **kwargs):
-        XMLBuilder.__init__(self, *args, **kwargs)
-        self._random_uuid = None
-
 
     ###################
     # Helper routines #
@@ -200,26 +219,21 @@ class Network(XMLBuilder):
         raise ValueError(_("Name '%s' already in use by another network." %
                          name))
 
-    def _get_default_uuid(self):
-        if self._random_uuid is None:
-            self._random_uuid = util.generate_uuid(self.conn)
-        return self._random_uuid
-
 
     ##################
     # XML properties #
     ##################
 
     _XML_ROOT_NAME = "network"
-    _XML_PROP_ORDER = ["ipv6", "name", "uuid", "forward",
+    _XML_PROP_ORDER = ["ipv6", "name", "uuid", "forward", "virtualport_type",
                        "bridge", "stp", "delay", "domain_name",
                        "macaddr", "ips", "routes", "bandwidth"]
 
     ipv6 = XMLProperty("./@ipv6", is_yesno=True)
     name = XMLProperty("./name", validate_cb=_validate_name)
-    uuid = XMLProperty("./uuid",
-                       validate_cb=lambda s, v: util.validate_uuid(v),
-                       default_cb=_get_default_uuid)
+    uuid = XMLProperty("./uuid")
+
+    virtualport_type = XMLProperty("./virtualport/@type")
 
     # Not entirely correct, there can be multiple routes
     forward = XMLChildProperty(_NetworkForward, is_single=True)
@@ -238,11 +252,11 @@ class Network(XMLBuilder):
 
     def add_ip(self):
         ip = _NetworkIP(self.conn)
-        self._add_child(ip)
+        self.add_child(ip)
         return ip
     def add_route(self):
         route = _NetworkRoute(self.conn)
-        self._add_child(route)
+        self.add_child(route)
         return route
 
 
@@ -261,7 +275,7 @@ class Network(XMLBuilder):
                 net.create()
             if autostart:
                 net.setAutostart(autostart)
-        except:
+        except Exception:
             net.undefine()
             raise
 
